.content {
  width: 50%;
  margin-left: 25%;
}

.hidden, .hidden * {
  display: none !important;
}

.fadein {
  transition: opacity 1s;
  opacity: 1;
  display: flex !important;
}

body form {
	box-shadow: none;
}
.new-folder {
  align-items: center;
  flex-wrap: wrap;
  border: none;
}
.new-folder > * {
  margin: 0px 10px;
}
.new-folder input {
  margin: 0px;
}
.new-folder input[type="submit"] {
  padding: 5px;
}

.delete-btn {
  padding: 0;
  border: none;
  background: none;
}
.delete-btn img {
  height: 40px;
}

.expand-btn {
  background: none;
  padding: 0;
  border: none;
  color: #920de9;
}

.folder-cont {
  border-left: 5px solid #ff4301;
  padding: 0px 10px;
  margin: 30px 0px;
}
.folder-cont h2,
.folder-cont p {
  margin: 5px;
}
.folder-cont ul {
  display: none;
  padding: 10px;
}
.folder-cont li {
  list-style: none;
  padding: 5px;
}
.folder-cont form {
  display: none;
}

.sidebar {
  margin: 0;
  padding: 0;
  width: 22%;
  position: fixed;
  height: 100%;
  overflow: auto;
}

.sidebar form {
  padding: 10px;
}

.sidebar form span {
  display: none;
}

.sidebar input[type="text"] {
  width: 40%;
  margin: 10px 0px;
}
.sidebar li {
  padding: 0;
  margin: 0;
}
.sidebar li a {
  margin: 0;
}

/* Active/current link */
.sidebar a.active {
  background-color: #4caf50;
  color: white;
}

mark {
	background-color: #f6efa6;
}

#pfp {
	margin: 15px;
}

/**
 * dark theme
 */

.toggle {
  font-size: 50px;
  /* color: #ffffff; */
  position: absolute;
  right: 90px;
  top: 10px;
}

.dark-theme {
  color: white;
  background: #151515;
}

.dark-theme button{
	color: #ed6663;
}
.dark-theme  a, .dark-theme code {
  color: #ffffff;
}

.dark-theme blockquote {
	color: #d1cebd;
}
.dark-theme input, .dark-theme select, .dark-theme textarea {
	border: none;
	background: #1f1f1f;
	color: white;

}
.dark-theme a:hover, .dark-theme a:visited {
  color: #ed6663;
}

.dark-theme .matches {
	background: #ed6663;
}

.dark-theme img#pfp {
    -webkit-filter: invert(100%);
    filter: invert(100%);
}

#pfp {
	margin: 15px;
}

#right-pane {
	width: 100%;
	margin: 0px;
}

.script-output {
	border: 3px;
	border-style: dashed;
	display: block;
	width: fit-content;
	min-width: 95%;
	unicode-bidi: embed;
	font-family: monospace;
	white-space: pre;
	margin-bottom: 1em;
	padding: 7px;
}

.LaTeX {
	width: 50px !important;
}
.LaTeX:before {
	content: "LaTeX"
}

#searchBar {
	border: 1px solid #dfe1e5; border-radius: 5px
}


@media only screen and (max-width: 500px) {
  .content {
    width: 90%;
    margin: auto;
  }
  .sidebar {
    display: none;
  }
}
