
{% extends "base.html" %}

{% block content %}
	<h1>{{ title }}</h1>
    <form action="" method="post" novalidate>
        {{ form.hidden_tag() }}
		{% for field in form %}
			{% if field.name != "csrf_token" and field.name != "submit" %}
				<p>
					{% for error in field.errors %}
						<span style="color: red;">{{ error }}</span>
					{% endfor %}
					{{ field.label }}<br>
					{{ field }}
				</p>
			{% endif %}
		{% endfor %}
        <p>{{ form.submit() }}</p>
    </form>
{% endblock %}
